//==========================================================================
// CSHAPE.CPP      Source code for the CShape class. 
//
// CREATED BY      George C Cross, for Synergy Solutions Inc
// CREATED ON      August 10 , 1995
// LAST MODIFIED   September 9, 1995. Added Color data members and functions. 
//
//==========================================================================

#include "cshape.h"


//==========================================================================
// Class constructor
//==========================================================================
CShape::CShape (int L, int T, int R, int B, 
                COLORREF CSelected, COLORREF CNotSelected)
{
 left = L; 
 top = T; 
 right = R; 
 bottom = B;
 ColorSelected = CSelected;
 ColorNotSelected = CNotSelected; 
 selected = FALSE;
} 


//==========================================================================
// FUNCTION        CShape::DrawShape ()
//
// PARAMETERS      dc        Device context in which the shape is to be drawn
//
// RETURNS         nothing
//
// REMARKS         Used to fill a circle with the appropriate color
//==========================================================================
void CShape::DrawShape (HDC hdc)
{
 HBRUSH hBrush,     //Brush used to color circle
        hOldBrush;  //Old brush of window's device context
 HPEN   hPen,       //Pen used to draw borders
        hPenColor,
        hOldPen;    //Old pen of window's device context
 
 //
 //If the shape is selected, then
 //
 if (Selected ()) 
    {
     //
     //Use color _________
     //
     hBrush = CreateSolidBrush (ColorSelected);
     hPenColor = CreatePen (PS_SOLID, 0, ColorSelected);
    }
 else   //Otherwise, 
    {
     //
     //Use color ________
     //
     hBrush = CreateSolidBrush (ColorNotSelected);
     hPenColor = CreatePen (PS_SOLID, 0, ColorNotSelected);
    }
 //
 //Insert the brush into the device context
 //
 hOldBrush = SelectObject (hdc, hBrush);

 //
 //Create the hightlight pen and insert it into the window's device context
 //
 hPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNHIGHLIGHT));
 hOldPen = SelectObject (hdc, hPen); 
 
 //
 //Draw the circle
 //
 Ellipse (hdc, left, top, right, bottom);
 
 //
 //Replace the original brush in the device context and free GDI resources
 //used by the brush we created
 //
 SelectObject (hdc, hOldBrush);
 DeleteObject (hBrush);
 
 //
 //Replace the original pen and free GDI resources used by the pen created
 //
 SelectObject (hdc, hOldPen);
 DeleteObject (hPen);

 //Get the highlight brush and insert it into the device context
 //
 hBrush = CreateSolidBrush (GetSysColor (COLOR_BTNHIGHLIGHT));
 hOldBrush = SelectObject (hdc, hBrush);
 
 //
 //Put the pen used to draw the shine on the light in to the device context
 //
 hOldPen = SelectObject (hdc, hPenColor);
 
 //
 //Get a rectangle in the upper left corner of the light.  This will be 
 //used to draw a "shine" on the light.
 //
 RECT rc;
 rc.left = left + (right - left) / 5;
 rc.top =  top  + (bottom - top) / 5;
 rc.right = rc.left + (right - left) /2;
 rc.bottom = rc.top + (bottom - top) /2;
 
 //
 //Draw the "shine" using the rectangle obtained above
 //
/*
 int width = right - left;
 int height = bottom - top;
 
 MoveTo (hdc, left + width /4 + 1, top + height /2 - 1);
 LineTo (hdc, left + width /4 + 3, top + height /2 - 2);
 LineTo (hdc, left + width /4 + 5, top + height /2 - 3);
*/
 Ellipse (hdc, 
		        rc.left,
		        rc.top,
		        rc.right,
		        rc.bottom);

 MoveTo (hdc, rc.left + 1, rc.top + 1);
 LineTo (hdc, rc.left + 2, rc.top + 2);
 MoveTo (hdc, rc.right - 2, rc.bottom - 2);
 LineTo (hdc, rc.right, rc.bottom);
 
 //
 //Replace the original brush in the device context and free GDI resources
 //used by the brush we created
 //
 SelectObject (hdc, hOldBrush);
 DeleteObject (hBrush);  
      
 //
 //Replace the original pen of the device context and free GDI resources
 //allotted to the new pen we created
 //
 SelectObject (hdc, hOldPen);
 DeleteObject (hPenColor);

 //-------------------------------------------------------------------------
 //Below we add a 3D look to the circle by drawing a shadow Arc around 
 //the circle
 //-------------------------------------------------------------------------
 //
 //Create the shadow pen and insert it into the window's device context
 //
 hPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNSHADOW));
 hOldPen = SelectObject (hdc, hPen);

 //
 //Draw the arc. Please see Windows SDK for an explanation of arguments
 //
 Arc (hdc, left, top, right, bottom, left, bottom, right, top);

 //
 //Replace the original pen of the device context and free GDI resources
 //allotted to the new pen we created
 //
 SelectObject (hdc, hOldPen);
 DeleteObject (hPen);

}   //End   void CShape::DrawShape (CDC &dc)
          
     
//==========================================================================
// FUNCTION        CShape::Offset ()
//
// PARAMETERS      x   units of horizontal shift (negative means shift left)
//                 y   units of vertical shift (negative means shift up)
//
// RETURNS         nothing
//
// REMARKS         Used to reposition the coordinates of CShape in a window
//==========================================================================
void CShape::Offset (int x, int y)
{
 left   = left   + x;
 right  = right  + x;
 top    = top    + y;
 bottom = bottom + y;
}


//==========================================================================
// FUNCTION        CShape::operator= ()
//
// PARAMETERS      srcRect     Original rectangle, coordinates of which, will
//                             be assigned to the CShape object
//
// RETURNS         nothing
//
// REMARKS         Overloaded operator to allow easy assignment of left,
//                 top, right, bottom to the CShape object
//==========================================================================
void CShape::operator= (const CShape& srcShape)
{
 left   = srcShape.left;
 top    = srcShape.top;
 right  = srcShape.right;
 bottom = srcShape.bottom;
 SetColors (srcShape.ColorSelected, srcShape.ColorNotSelected);
 //selected = srcShape.Selected ();
}

//==========================================================================
// FUNCTION       CShape::SetColors ()
//
// PARAMETERS     CSelected      Color to display when shape is selected
//                CNotSelected   Color to display when shape is not selected
//
// RETURNS        nothing
//
// REMARKS        Used to assign the colors of the shape when selected and
//                not selected.
//==========================================================================
void CShape::SetColors (COLORREF CSelected, COLORREF CNotSelected)
{
 ColorSelected = CSelected; 
 ColorNotSelected = CNotSelected;
}

