//==========================================================================
// HOOKFNS.CPP     Source file for our custom functions which will be "hooked
//                 into" specified API functions. Our custom functions will 
//                 thereby be called when any other application makes calls
//                 to one of the API functions
//
// CREATED BY      George C Cross, for Synergy Solutions Inc
// CREATED ON      September 6, 1995
// LAST MODIFIED   August 26, 1995.  Revised WriteComm & ReadComm to improve
//                 RX and TX light response.
//
// (c) Copyright Synergy Solutions Inc, 1995. All rights reserved.
//==========================================================================

#include "hookfns.h"   //Header to this source file
#include <windows.h>   //Windows API header used for HDC in MGetCommError
#include "mainwnd.h"   //Used only for OnCommNotify function
#include "mlights.h"   //Used for constants and global variables 
#include "prochook.h"  //Used for PROCHOOK.LIB functions
#include "cshape.h"    //Header for the CSHAPE object used in MGetCommError



//==========================================================================
// Global variables for this module
//========================================================================== 

extern HWND        ghWnd;                      //Handle to the main window
extern NPHOOKCHILD                             //Array of near pointers to 
       npHookedFunctions[NUM_HOOKEDFUNCTIONS]; //hooked function structures
extern CShape Lights[NUM_LIGHTS];              //Modem light window objects
extern UINT gEventWord;     //Comm device event word from GetCommEventMask() 
extern int    gPortID;      //Comm device ID returned from OpenComm()
extern BOOL   gbPortOpened; //Flag indicating port is open or closed
extern UINT   gwOptions;    //Flag indicating the window's options

extern COMSTAT gComStatBuf; //Windows API structured variable containing
                            //status information of a communications device 
extern BOOL       gbRemoveCaptionOnRestore;//Flag to remove caption on restore


extern void AddCaption (HWND hWnd);   //For adding the caption on minimize

//==========================================================================
// FUNCTION        MOpenComm ()
//
// PARAMETERS      lpszDevControl     Address of device control information 
//                 cbInQueue          Size of receiving buffer
//                 cbOutQueue         Size of transmitting buffer
//
// RETURNS         int     An identification of the device if the port was
//                         opened. A negative number if unsuccessful.
//
// REMARKS         Our custom function displays the port ID on the window
//                 caption after the original API function has been called.
//                 See Windows API documentation for more information.
//==========================================================================
int __export WINAPI MOpenComm (LPCSTR lpszDevControl,
                               UINT   cbInQueue,
                               UINT   cbOutQueue)
{
 char szString[12];     //String to hold new window caption text
 
 HWND hWnd = GetActiveWindow (); //Handle to the active window of the 
                                 //application which called OpenComm () 
                                 //original API function
 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[OPENCOMM]);
 
 //
 //Call the original function, which returns an identifier to the
 //communications device opened
 //
 gPortID = OpenComm (lpszDevControl,
                     cbInQueue,
                     cbOutQueue);
    
 //
 //Rehook the function
 //
 ProcHook (npHookedFunctions[OPENCOMM]);
 

 //
 //Check that the call was successful, if so, then...
 //
 if (gPortID > 0)
    {
     //
     //Indicate that the port has now been opened
     //
     gbPortOpened = TRUE;

     //
     //Display the Port ID in the main window's caption
     //
     wsprintf ((LPSTR) szString, "COM%d:", gPortID + 1);
     SetWindowText (ghWnd, (LPSTR) szString); 
     
     //
     //Specify which communication events we would like to enable
     //for this communication device event word
     //
     SetCommEventMask (gPortID, EV_TXEMPTY);
    
     //
     //For the communications device returned from OpenComm, request 
     //communications event nofication to our window when the event word changes.
     //See Windows SDK help for an explanation of EnableCommNotification()
     //
     EnableCommNotification (gPortID,   //Communications device ID from OpenComm()
                             ghWnd,     //Handle to the main MLights window
                             -1,        //No CN_RECEIVE events
                             -1);       //No CN_TRANSMIT events                                 
    
    
     //
     //Set a timer to periodically check the status of this communications device
     //
     SetTimer (ghWnd, ID_MLIGHTSTIMER, 500, NULL);
     
     //
     //Restore the our MLIGHTS main window if it is minimized
     //
     ShowWindow (ghWnd, SW_RESTORE);
     
     //
     //Keep the main window active in the communications application which made
     //the OpenComm() original API function call
     //
     SetActiveWindow (hWnd);
    
     //
     //Change the necessary lights at this time by calling OnCommNotify.
     //This is probably cheating since OnCommNotify is a message response
     //function.  But I'm a hack and this works.
     //
     OnCommNotify ();
    } //End  if (gPortID > 0)
     
 //
 //Return the identifier for the communications device opened, as
 //returned by the original API function OpenComm
 //
 return gPortID;                   
}  //End   int __export WINAPI MOpenComm                                          


//==========================================================================
// FUNCTION        MCloseComm ()
//
// PARAMETERS      idComDev        ID of port to close
//
// RETURNS         int     Zero if close was successful. Negative otherwise
//
// REMARKS         Our custom function replaces the main window caption 
//                 with the default text.
//                 See Windows API documentation for more information.
//==========================================================================
int __export WINAPI MCloseComm (int idComDev)
{
 int PortClosed;   //Return value for the original API function
 
 KillTimer (ghWnd, ID_MLIGHTSTIMER); //Turn the timer bit off in msg queue

 //
 //Indicate that the port is now closed
 //
 gbPortOpened = FALSE;
 
 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[CLOSECOMM]);
 
 //
 //Call the original API function
 //
 PortClosed = CloseComm (idComDev);
 
 //
 //Re hook our custom function
 //
 ProcHook (npHookedFunctions[CLOSECOMM]);
 
 //
 //Display default main window caption
 //
 SetWindowText (ghWnd, APP_TITLE); 
 
 //
 //Turn off all the lights
 //
 HDC hdc = GetDC (ghWnd);
 for (int iloop = 0; iloop < NUM_LIGHTS; iloop++)
    {
     Lights[iloop].Select (FALSE);
     Lights[iloop].DrawShape (hdc);
    }
 ReleaseDC (ghWnd, hdc);   

 //
 //Minimize the window if the MINIFPORTCLOSED bit flag is on in the
 //options word
 //            
 if (gwOptions & MINIFPORTCLOSED)
    {
     if (gwOptions & HIDECAPTION)
        {
         AddCaption (ghWnd);
        } 
     ShowWindow (ghWnd, SW_SHOWMINNOACTIVE);  //Minimize it !
     gbRemoveCaptionOnRestore = TRUE;
    }
     
 //
 //Return the value returned by the original API function
 //
 return PortClosed;                   
}  //End  int __export WINAPI MCloseComm (int idComDev) 


//==========================================================================
// FUNCTION        MGetCommError ()
//
// PARAMETERS      idComDev        ID of port accessed
//                 lpStat          pointer to the device status buffer
//
// RETURNS         int     Error value of most recent communications function
//                         call to the device idComDev.
//
// REMARKS         Our custom function colors the main window "lights" 
//                 according to the values in lpStat.
//                 See Windows API documentation for more information.
//==========================================================================
int __export WINAPI MGetCommError (int idComDev, COMSTAT FAR* lpStat)
{
 int CommError;    //Return value of the original API function

 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[GETCOMMERROR]);
 
 //
 //Call the original API function
 //
 CommError = GetCommError (idComDev, lpStat);
 
 //
 //Re hook our custom function
 //
 ProcHook (npHookedFunctions[GETCOMMERROR]);

 //
 //Return the value returned by the original API function
 //
 return CommError;                                             
}  // End    int __export WINAPI MGetCommError (int, COMSTAT FAR*)


//==========================================================================
// FUNCTION        MReadComm ()
//
// PARAMETERS      idComDev        ID of communications device accessed
//                 lpvBuf          pointer to the buffer for read bytes
//                 cbRead          number of bytes to read
//
// RETURNS         int     Number of bytes read if successful. Otherwise
//                         a negative number, the absolute of which is 
//                         the number of bytes read.
//
// REMARKS         Our custom function turns on the RX light during reading
//                 of bytes from the communications device idComDev.
//                 See Windows API documentation for more information.
//==========================================================================
int __export WINAPI MReadComm(int idComDev, void FAR* lpvBuf, int cbRead)
{
 int nBytesRead;          //Return value of ReadComm () = bytes actually read

 //
 //Retrieve a buffer filled with status information for this 
 //communcations device
 //
 GetCommError (gPortID, &gComStatBuf);
 
 //
 //If the presence or absence of characters in the receive queue 
 //does not correspond with the RX light being ON or OFF respectively, then
 // 
 if ((gComStatBuf.cbInQue > 0) != (Lights[RX].Selected ()))
    {
     Lights[RX].Select (gComStatBuf.cbInQue > 0); //Synchronize the RX light
     HDC hdc = GetDC (ghWnd);                  //Get a device context handle
     Lights[RX].DrawShape (hdc);               //Paint the light to update it
     ReleaseDC (ghWnd, hdc);                   //Release the dc handle
    }

 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[READCOMM]);
 
 //
 //Call the original API function
 //
 nBytesRead = ReadComm (idComDev, lpvBuf, cbRead);
 
 //
 //Re hook our custom function
 //
 ProcHook (npHookedFunctions[READCOMM]);
 
 //
 //Return the value returned by the original API function
 //
 return nBytesRead;                                             
}   //End  int __export WINAPI MReadComm ()


//==========================================================================
// FUNCTION        MWriteComm ()
//
// PARAMETERS      idComDev        ID of communications device accessed
//                 lpvBuf          pointer to the buffer for written bytes
//                 cbWrite         number of bytes to write
//
// RETURNS         int     Number of bytes written if successful. Otherwise
//                         a negative number, the absolute of which is 
//                         the number of bytes written.
//
// REMARKS         Our custom function turns on the TX light during writting
//                 of bytes to the communications device idComDev.
//                 .
//                 See Windows API documentation for more information.
//==========================================================================
int __export WINAPI MWriteComm(int idComDev, 
                               void const FAR* lpvBuf, 
                               int cbWrite)
{
 int nBytesWritten;       //Return value of WriteComm () = bytes written
 
 //
 //If the object emulating the TX light is not selected, the TX light is
 //therefore OFF, so turn it ON by, 
 //
 if (Lights[TX].Selected () == FALSE)
    {
     Lights[TX].Select (TRUE);   //Select the object emulating the TX light
     HDC hdc = GetDC (ghWnd);    //Get a handle to the device context
     Lights[TX].DrawShape (hdc); //Paint the object to show the light ON
     ReleaseDC (ghWnd, hdc);     //Release the device context handle
    }
    
 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[WRITECOMM]);
 
 //
 //Call the original API function
 //
 nBytesWritten = WriteComm (idComDev, lpvBuf, cbWrite);
 
 //
 //Re hook our custom function
 //
 ProcHook (npHookedFunctions[WRITECOMM]);

 //
 //Return the value returned by the original API function
 //
 return nBytesWritten;                                             
} //End  int __export WINAPI MWriteComm ()


//==========================================================================
// FUNCTION        MSetCommEventMask ()
//
// PARAMETERS      idComDev        ID of communications device accessed
//                 fuEvtMask       Events to enable in the event word
//
// RETURNS         UINT FAR*     Pointer to the event word of this comm
//                               device. 
//
// REMARKS         Our custom function turns ensures that the EV_RXCHAR
//                 flag is always enabled.
//                 See Windows API documentation for more information.
//==========================================================================
UINT FAR* __export WINAPI MSetCommEventMask(int idComDev, UINT fuEvtMask) 
{
 UINT FAR* lpnEventWord;    //Pointer returned by the original API function
 //
 //Ensure the event mask has EV_RXCHAR and EV_TXEMPTY set
 //
 fuEvtMask = fuEvtMask | EV_TXEMPTY;
 
 //
 //Unhook temporarily to call the original function
 //
 ProcUnhook (npHookedFunctions[SETCOMMEVENTMASK]);
 
 //
 //Call the original API function
 //
 lpnEventWord = SetCommEventMask (idComDev, fuEvtMask);
 
 //
 //Re hook our custom function
 //
 ProcHook (npHookedFunctions[SETCOMMEVENTMASK]);

 //
 //Return the value returned by the original API function
 //
 return lpnEventWord;
} //End  UINT FAR* __export WINAPI MSetCommEventMask();                                              


//==========================================================================
// FUNCTIONS       GetCommMSRShadow ()
//
// PARAMETERS      idComDev     ID of communications device accessed
//
// RETURNS         LPBYTE       Pointer to the byte containing modem
//                              status bits
//
// REMARKS         Not documented in the Windows API, COMM.DRV keeps a byte
//                 reflecting the modem status bits CTS, DSR, RI, RLSD at
//                 the 36th byte past the event word for the communications
//                 device.
//                 See Microsoft Knowledge Base article Q101417
//
//==========================================================================   
LPBYTE GetCommMSRShadow (short idComDev) 
{ 
   //
   //Return far pointer to MSR shadow byte 
   //
   return (((LPBYTE)SetCommEventMask(idComDev, 0) + COMM_MSRSHADOW)); 

} //End LPBYTE GetCommMSRShadow (short idComDev)


 